#define  WANTVXDWRAPS

#include <basedef.h>
#include <vmm.h>
#include <vmmreg.h>
#include <vxdwraps.h>
#include <configmg.h>
#include <vmmreg.h>
#include <regstr.h>
#include <debug.h>

#include "loadsw32.h"
#include "sw32dsp.h"
#include "mydebug.h"

//  Helper macros
//
#define NUM_ELEMENTS(array)     (sizeof(array) / sizeof(array[0]))

//  Global variables
//
WORD awDmaSpecs[] =  { DMA0, DMA1, DMA3, DMA5, DMA6, DMA7 };
WORD awValidDmas[] = { 0, 1, 3, 5, 6, 7 };
WORD awIrqSpecs[] =  { IRQ3, IRQ5, IRQ7, IRQ9, IRQ10, IRQ11, IRQ12 };
WORD awValidIrqs[] = { 3, 5, 7, 9, 10, 11, 12 };
LONG lNumInPorts = 0;
LONG lNumOutPorts = 0;
INT nPseudoFilePos = 0;

//	Globals for the SoundWave32 settings
//	NOTE: These values are short integers to be compatible with the 16-bit 
//	driver (sw32wss.drv), which maintains the registry settings.
SHORT nCdRomPort = 0;
SHORT nCdRomIRQ = 0;
SHORT nCdRomDMA = 0;
SHORT nSbPort = 0x220;
SHORT nSbIRQ = 7;
SHORT nSbDMA = 1;
SHORT nMpuPort = 0;
SHORT nMpuIRQ = 0;
SHORT nGamePort = 1;


WORD InPortWord (WORD wAddress)
{
    WORD wResult;
    _asm {
        mov DX, wAddress
        in  AX, DX
        mov wResult, AX
        }
    lNumInPorts++;

    return (wResult);       
}


VOID OutPortWord (WORD wAddress, WORD wValue)
{
    WORD wResult;
    _asm {
        mov DX, wAddress
        mov AX, wValue
        out DX, AX
        mov wResult, AX
        }
    lNumOutPorts++;
}


//  Pseudo-file operations
//

FILE* OpenBinaryFile (PCSZ pszFile)
{
    nPseudoFilePos = 0;
    return (0);
}

BYTE GetNextByte (FILE* fIn)
{
    //return (bDebugLoad[nPseudoFilePos++]);
    return (abGenmidLoad[nPseudoFilePos++]);
}


BOOL EndOfFile (FILE* fIn)
{
    //return (nPseudoFilePos > nMAX_POS);
    return (nPseudoFilePos > nGenmidLoadSize);
}

VOID CloseBinaryFile (FILE* fIn)
{
}


WORD GetDmaSpec (WORD wDma)
{
    WORD wDmaSpec = 0;
    INT i;
    ASSERT(NUM_ELEMENTS(awValidDmas) == NUM_ELEMENTS(awDmaSpecs));

    for (i = 0; i < NUM_ELEMENTS(awValidDmas); i++) {
        if (awValidDmas[i] == wDma) {
            wDmaSpec = awDmaSpecs[i];
            break;
            }
        }
    ASSERT(wDmaSpec != (WORD)-1);
    
    return (wDmaSpec);  
}


WORD GetIrqSpec (WORD wIrq)
{
    WORD wIrqSpec = 0;
    INT i;
    ASSERT(NUM_ELEMENTS(awValidIrqs) == NUM_ELEMENTS(awIrqSpecs));

    for (i = 0; i < NUM_ELEMENTS(awValidIrqs); i++) {
        if (awValidIrqs[i] == wIrq) {
            wIrqSpec = awIrqSpecs[i];
            break;
            }
        }
    ASSERT(wIrqSpec != (WORD)-1);
    
    return (wIrqSpec);  
}


//	Initialize the SoundWave32 DSP for emulating the Windows Sound System standard,
//	along with support for Sound Blaster and MIDI emulation.
//
//	NOTE: This procedure reflects the specifications found in Analog Devices'
//		  documentation for the Personal Sound System (the 1848, 2115, etc chipset).
//		  This documentation currently is available via FTP (ftp://ftp.analog.com).
//
InitSw32 (BOOL bForceReset, PCSZ pszFile, WORD wWssBase, WORD wIrq, WORD wDma)
{
    int i,j;
    long lCount=0;
    WORD wPortOffset=0;		// offset from 0x220 of SB emultation port (0x0 or 0x20)
    WORD wByte;				// byte read from the load file
    FILE* fIn;				// psuedo-file (for compatibility w/ DOS version)
	WORD wConfigSpec;		// encoding for configuration specification

    MYDPF("InitSw32: In");

    // Add support for Revision 'B' PSS ASIC
    MYDPF("Checking port signature");
    if((InPortWord(PSS_ID_VERS) != 0x4501) && (InPortWord(PSS_ID_VERS) != 0x4502)) {
        wPortOffset=0x20;
               
        if((InPortWord(PSS_ID_VERS + wPortOffset) != 0x4501) 
                && (InPortWord(PSS_ID_VERS + wPortOffset) != 0x4502)) {  
            MYDPF( "No SoundWave32/GameWave32 was found!");
            return (FALSE);
            }
        }
   
    MYDPF("Opening 'file'");
    fIn = OpenBinaryFile(pszFile);

	//	Display what we will be doing (to the kernel debugging stream and/or mono monitor)
	//	NOTE: wIrq and wDma are not specified in any of the port registers
	//		  so these parameters can be removed
    MYDPF("Setting WSS, SB & MIDI configs");
    MYDPF2("\tWSS at #AX; IRQ #DX", wWssBase, wIrq);
    MYDPF1("\t\tDMA = #AX", wDma);
    MYDPF1("\tDMA Spec is #AX", GetDmaSpec(wDma));
	MYDPF2("\tMIDI at #AX; IRQ #DX", nMpuPort, nMpuIRQ);
	MYDPF2("\tCD at #AX; IRQ #DX", nCdRomPort, nCdRomIRQ);
	MYDPF2("\tSB at #AX; IRQ #DX", nSbPort, nSbIRQ);
	MYDPF1("\t\tDMA at #AX", nSbDMA);

	//	Write the encoding for the Windows Sound System base address
    OutPortWord(WSS_CONFIG + wPortOffset, wWssBase << 4);  // eg, set WSS to 530h (5300)

	//	Write the SB emulation IRQ and DMA encoding
	//	NOTE: invalid values will be encoded as zeroes, which means disabled
	wConfigSpec = GetIrqSpec(nSbIRQ) | GetDmaSpec(nSbDMA);
    OutPortWord(SB_CONFIG + wPortOffset, wConfigSpec);
    MYDPF1("SB_CONFIG set to #AX", wConfigSpec)

	//	Write the MIDI emulation port and IRQ encoding
	wConfigSpec = ((WORD)nMpuPort << 4) | GetIrqSpec(nMpuIRQ);
    OutPortWord(MIDI_CONFIG + wPortOffset, wConfigSpec);
    MYDPF1("MIDI_CONFIG set to #AX", wConfigSpec);

	//	Write the CD interface port and IRQ encoding
	wConfigSpec = ((WORD)nCdRomPort << 4) | GetIrqSpec(nCdRomIRQ);
    OutPortWord(CD_CONFIG + wPortOffset, wConfigSpec);
    MYDPF1("CD_CONFIG set to #AX", wConfigSpec);

	//	Reboot the DSP   
    MYDPF("Sending reboot");
    wByte = GetNextByte(fIn);                   // get first byte of boot sequence
    OutPortWord(PSS_DATA + wPortOffset,0x00fe);	// send reboot command to existing software
   
    if (bForceReset) {
       i=2048;
       }
    else {
       for(i=0; i<2048; i++) {
           if(InPortWord(PSS_DATA + wPortOffset) == 0x5555)
               break;
           }
       }
   
    MYDPF("Sending reset");
    OutPortWord(PSS_DATA + wPortOffset, wByte);          // latch out first byte of boot
    if(i==2048) {
        OutPortWord(PSS_CONTROL + wPortOffset,0x2000);   // reset the DSP
        for (i=0; i<32767; i++) {
            ;
            }
        OutPortWord(PSS_CONTROL + wPortOffset,0x0000);   // set the DSP reset line high
        MYDPF("Hard reset performed.");
        }
    else {
        MYDPF("Hard reset not performed.");
        }
   
    MYDPF("Loading 'file'");
    for (i=1; i<=3072; i++) {
        for(j=0; j<2048; j++) {
            if(InPortWord(PSS_STATUS + wPortOffset) & PSS_FLAG3) {
                break;
                }
            }
        if(j==2048) {
            MYDPF("TIMEOUT ERROR after x bytes");
            return (FALSE);
            }
        OutPortWord(PSS_DATA + wPortOffset, GetNextByte(fIn));
        lCount++;
        if(i==3072) {
            for(j=0; j<2048; j++) {
                if(InPortWord(PSS_STATUS + wPortOffset) & PSS_FLAG3) {
                    break;
                    }
                }
            if(j==2048) {
                MYDPF("TIMEOUT ERROR after x bytes");
                return (FALSE);
                }
            wByte = GetNextByte(fIn);
            OutPortWord(PSS_DATA + wPortOffset, wByte);
            i=0;
            }
        if(EndOfFile(fIn)) {
            break;
            }
        }
    CloseBinaryFile(fIn);

    MYDPF("Setting misc stuff (eg, game support)");
    OutPortWord(PSS_DATA + wPortOffset,0);

	//	Enable the game port
	wConfigSpec = InPortWord(PSS_CONFIG + wPortOffset);
	if (nGamePort) {
    	MYDPF1("Setting game support on (#AX)", wConfigSpec | GAME_BIT);
		OutPortWord(PSS_CONFIG + wPortOffset, wConfigSpec | GAME_BIT);
		}
	else {
    	MYDPF1("Setting game support off (#AX)", wConfigSpec & ~GAME_BIT);
		OutPortWord(PSS_CONFIG + wPortOffset, wConfigSpec & ~GAME_BIT);
		}

   
    while (!(InPortWord(PSS_STATUS + wPortOffset) & PSS_READ_FULL)) {
        ;
        }
    wByte = InPortWord(PSS_DATA + wPortOffset);
    
    //  Tell them how things went   
    MYDPF1("Version is #AX", wByte);
    wByte = InPortWord(PSS_STATUS + wPortOffset);
    MYDPF1("Status = #AX", wByte);
    MYDPF1("Number of IN's from ports = #EAX", lNumInPorts);
    MYDPF1("Number of OUT's to ports = #EAX", lNumOutPorts);

    MYDPF("InitSw32: Out");

#if 1
	//	Wait a little for the chip to initialized
	VTD_Delay(500000);		// 1/2 second for good measure
#endif

    return (TRUE);
}
