#define _WINDOWS 1
#define _SNDSYS 1

#if defined(_WINDOWS)
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

#if defined(MSSNDSYS)
extern "C" {
#define Not_VxD
#include <vmm.h>
#include <configmg.h>
#include <mmsystem.h>
#include <mmddk.h>
#include <mmreg.h>
#include "config.h"
#define NOSTR
#include "mssndsys.h"
#include <verinfo.h>
#include <wsshlpid.h>
#include "driver.h"
}
#endif

#include "loadsw32.h"
#include "mydebug.h"
#if defined(DEBUG)
char szDebugOut[256];
#endif


//	Global variables
//
WORD awDmaSpecs[] =  { DMA0, DMA1, DMA3, DMA5, DMA6, DMA7 };
WORD awValidDmas[] = { 0, 1, 3, 5, 6, 7 };
WORD awIrqSpecs[] =  { IRQ3, IRQ5, IRQ7, IRQ9, IRQ10, IRQ11, IRQ12 };
WORD awValidIrqs[] = { 3, 5, 7, 9, 10, 11, 12 };
LONG lNumInPorts = 0;
LONG lNumOutPorts = 0;
CHAR szRB[] = "rb";
CHAR szFile[256];
CHAR BCODE szTHE_FILE[] = "C:\\WIN95\\SYSTEM\\GENMID.LD";	// default file



//	Helper macros
//
#define NUM_ELEMENTS(array)     (sizeof(array) / sizeof(array[0]))


//	Helper (inline) functions
//	
INLINE WORD InPortWord (WORD wAddress)
{
    WORD wResult;
    lNumInPorts++;
    _asm {
        mov DX, wAddress
        in  AX, DX
        mov wResult, AX
        }

    return (wResult);       
}


INLINE VOID OutPortWord (WORD wAddress, WORD wValue)
{
    lNumOutPorts++;
    WORD wResult;
    _asm {
        mov DX, wAddress
        mov AX, wValue
        out DX, AX
        mov wResult, AX
        }
}


WORD GetDmaSpec (WORD wDma)
{
    WORD wDmaSpec = 0;
    INT i;
    ASSERT(NUM_ELEMENTS(awValidDmas) == NUM_ELEMENTS(awDmaSpecs));

    for (i = 0; i < NUM_ELEMENTS(awValidDmas); i++) {
        if (awValidDmas[i] == wDma) {
            wDmaSpec = awDmaSpecs[i];
            break;
            }
        }
    ASSERT(wDmaSpec != (WORD)-1);
    
    return (wDmaSpec);  
}


WORD GetIrqSpec (WORD wIrq)
{
    WORD wIrqSpec = 0;
    INT i;
    ASSERT(NUM_ELEMENTS(awValidIrqs) == NUM_ELEMENTS(awIrqSpecs));

	for (i = 0; i < NUM_ELEMENTS(awValidIrqs); i++) {
        if (awValidIrqs[i] == wIrq) {
            wIrqSpec = awIrqSpecs[i];
            break;
            }
        }
    ASSERT(wIrqSpec != (WORD)-1);
    
    return (wIrqSpec);  
}


FILE* OpenBinaryFile (PCSZ pszFile)
{
    lstrcpy(szFile, pszFile);
    DEBUG_OUT(szFile);
    return (fopen(szFile, szRB));
}


INLINE BYTE GetNextByte (FILE* pFile)
{
    return (fgetc(pFile));
}


BOOL EndOfFile (FILE* pFile)
{
    return (feof(pFile));
}


VOID CloseBinaryFile (FILE* fIn)
{
    fclose(fIn);
}





//	Initialize the SoundWave32 DSP for emulating the Windows Sound System standard,
//	along with support for Sound Blaster and MIDI emulation.
//
//	NOTE: This procedure reflects the specifications found in Analog Devices'
//		  documentation for the Personal Sound System (the 1848, 2115, etc chipset).
//		  This documentation currently is available via FTP (ftp://ftp.analog.com).
//
extern "C" BOOL WINAPI
InitSw32 (BOOL bForceReset, PCSZ pszFile, WORD wWssBase, WORD wIrq, WORD wDma, LPDRVDATA pDrvData)
{
	DEBUG_OUT6("InitSw32(%d, %s, %04x, %d, %d, 0x%08lx)", bForceReset, (LPSTR)pszFile, wWssBase, wIrq, wDma, pDrvData);

   	FILE *fIn;
   	int i,j;					// file with DSP code
   	long lCount=0;				// count of bytes loaded
   	WORD wPortOffset=0;			// offset from 0x220 of SB emultation port (0x0 or 0x20)
	WORD wConfigSpec;			// encoding for configuration specification

   	// Reset the global counters
   	lNumInPorts = 0;
   	lNumOutPorts = 0;

    // Add support for Revision 'B' PSS ASIC
    DEBUG_OUT("Checking port signature" );
    if((InPortWord(PSS_ID_VERS) != 0x4501) && (InPortWord(PSS_ID_VERS) != 0x4502)) {
		wPortOffset=0x20;
               
		if((InPortWord(PSS_ID_VERS + wPortOffset) != 0x4501) && (InPortWord(PSS_ID_VERS + wPortOffset) != 0x4502)) {  
            DEBUG_OUT("No SoundWave32/GameWave32 was found!");
            return (FALSE);
            }
        }
   
    DEBUG_OUT("Opening file" );
    if ((fIn = OpenBinaryFile(pszFile)) == NULL) {
        DEBUG_OUT1("cannot open %s", (LPSTR)pszFile);
        return (FALSE);
        }

	//	Display what we will be doing (to the kernel debugging stream and/or mono monitor)
	//	NOTE: wIrq and wDma are not specified in any of the port registers
	//		  so these parameters can be removed
   	DEBUG_OUT("Setting WSS, SB & MIDI configs");
    DEBUG_OUT3("\tWSS at %04x; IRQ %u; DMA = %u", wWssBase, wIrq, wDma);
	DEBUG_OUT2("\tMIDI at %04x; IRQ %d", pDrvData->nMpuPort, pDrvData->nMpuIRQ);
	DEBUG_OUT2("\tCD at %04x; IRQ %d", pDrvData->nCdRomPort, pDrvData->nCdRomIRQ);
	DEBUG_OUT3("\tSB at %04x; IRQ %d; DMA: %d", pDrvData->nSbPort, pDrvData->nSbIRQ, pDrvData->nSbDMA);

	//	Write the encoding for the Windows Sound System base address
    OutPortWord(WSS_CONFIG + wPortOffset, wWssBase << 4);  // eg, set WSS to 530h (5300)

	//	Write the SB emulation IRQ and DMA encoding
	//	NOTE: invalid values will be encoded as zeroes, which means disabled
	wConfigSpec = GetIrqSpec(pDrvData->nSbIRQ) | GetDmaSpec(pDrvData->nSbDMA);
    OutPortWord(SB_CONFIG + wPortOffset, wConfigSpec);
    DEBUG_OUT1("SB_CONFIG set to 0x%x", wConfigSpec)

	//	Write the MIDI emulation port and IRQ encoding
	wConfigSpec = ((WORD)pDrvData->nMpuPort << 4) | GetIrqSpec(pDrvData->nMpuIRQ);
    OutPortWord(MIDI_CONFIG + wPortOffset, wConfigSpec);
    DEBUG_OUT1("MIDI_CONFIG config set to 0x%x", wConfigSpec)

	//	Write the CD interface port and IRQ encoding
	wConfigSpec = ((WORD)pDrvData->nCdRomPort << 4) | GetIrqSpec(pDrvData->nCdRomIRQ);
    OutPortWord(CD_CONFIG + wPortOffset, wConfigSpec);
    DEBUG_OUT1("CD_CONFIG config set to 0x%x", wConfigSpec)
   
    DEBUG_OUT("Sending reboot" );
    j=GetNextByte(fIn);                                 // get first byte of boot sequence
    OutPortWord(PSS_DATA + wPortOffset,0x00fe);         // send reboot command to existing software //
   
	//	Reboot the DSP   
    if (bForceReset) {
       i=2048;
       }
    else {
       for(i=0; i<2048; i++) {
               if(InPortWord(PSS_DATA + wPortOffset) == 0x5555)
               break;
           }
       }
   
    DEBUG_OUT("Sending reset" );
        OutPortWord(PSS_DATA + wPortOffset,j);             // latch out first byte of boot
    if(i==2048) {
        OutPortWord(PSS_CONTROL + wPortOffset,0x2000); // reset the DSP
        for (i=0; i<32767; i++) {
            ;
            }
        OutPortWord(PSS_CONTROL + wPortOffset,0x0000); // set the DSP reset line high
        DEBUG_OUT("Hard reset performed.");
        }
    else {
            DEBUG_OUT("Hard reset not performed.");
        }
   
    DEBUG_OUT("Loading file" );
    for (i=1; i<=3072; i++) {
        for(j=0; j<2048; j++) {
                if(InPortWord(PSS_STATUS + wPortOffset) & PSS_FLAG3) {
                break;
                }
            }
        if(j==2048) {
                DEBUG_OUT1("TIMEOUT ERROR after %ld bytes!!\b", lCount);
            return (FALSE);
            }
            OutPortWord(PSS_DATA + wPortOffset,GetNextByte(fIn));
        lCount++;
        if(i==3072) {
            for(j=0; j<2048; j++) {
                    if(InPortWord(PSS_STATUS + wPortOffset) & PSS_FLAG3) {
                    break;
                    }
                }
            if(j==2048) {
                    DEBUG_OUT1("TIMEOUT ERROR after %ld bytes!!!\b", lCount);
                return (FALSE);
                }
            i=GetNextByte(fIn);
            OutPortWord(PSS_DATA + wPortOffset,i);
            i=0;
            }
        if(EndOfFile(fIn)) {
            break;
            }
        }
        CloseBinaryFile(fIn);

    OutPortWord(PSS_DATA + wPortOffset,0);

	//	Enable the game port
	wConfigSpec = InPortWord(PSS_CONFIG + wPortOffset);
	if (pDrvData->nGamePort) {
    	DEBUG_OUT1("Setting game support on (%04X)", wConfigSpec | GAME_BIT);
		OutPortWord(PSS_CONFIG + wPortOffset, wConfigSpec | GAME_BIT);
		}
	else {
    	DEBUG_OUT1("Setting game support off (%04X)", wConfigSpec & ~GAME_BIT);
		OutPortWord(PSS_CONFIG + wPortOffset, wConfigSpec & ~GAME_BIT);
		}
   
   
    while (!(InPortWord(PSS_STATUS + wPortOffset) & PSS_READ_FULL)) {
        ;
        }
    i = InPortWord(PSS_DATA + wPortOffset);
   

    //  Tell them how things went   
    DEBUG_OUT("Displaying status");
    DEBUG_OUT3("%s Version %d.%02d", (PCSZ)pszFile, i/256, i%256);
    DEBUG_OUT2("%ld words loaded into PSS at %xh.", lCount/3, 0x220 + wPortOffset);
    DEBUG_OUT2("%ld in-ports; %ld out-ports", lNumInPorts, lNumOutPorts);

    WORD wStatus = InPortWord(PSS_STATUS + wPortOffset);
    DEBUG_OUT1("Status is: %x", wStatus);
    
    DEBUG_OUT("InitSw32: Out" );

    return (TRUE);
}
