//---------------------------------------------------------------------------
//
//  Module:   proppage.c
//
//  Description:
//      Property page handling code.  Please note that the
//      property sheet that is displayed is the dynamic
//      based on whether the Sound Blaster device is
//      software emulated or it is available in hardware.
//
//
//---------------------------------------------------------------------------
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1995 Microsoft Corporation.  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <windows.h>
#include <windowsx.h>
#include <ctype.h>

// stop compiler from griping about in-line

#pragma warning (disable:4704)

#define  Not_VxD
#include <vmm.h>
#include <configmg.h>

#include <mmsystem.h>
#include <mmddk.h>
#include <mmreg.h>
#include "config.h"
#define NOSTR
#include "mssndsys.h"
#include "driver.h"
#include "mixer.h"

#include <verinfo.h>
#include <wsshlpid.h>

const static DWORD aHelpIds[] = {  // Context Help IDs
    IDC_DRVPROP_EMULATION_GRP,      IDH_COMM_GROUPBOX,
    IDC_DRVPROP_SINGLEMODEDMA,      IDH_WINDOWS_SNDSYS_DMA,       
    IDC_DRVPROP_ACCEPTCLOSERATES,   IDH_WINDOWS_SNDSYS_SAMPLE,    
    IDC_DRVPROP_ADVANCED_GRP,       IDH_COMM_GROUPBOX,
    IDC_DRVPROP_ENABLESB,           IDH_WINDOWS_SNDSYS_SNDBLASTER,
    0, 0
};

static const char cszHelpFile[] = "MMDRV.HLP";

static char BCODE gszRegKeyConfig[]           = REGSTR_SUBKEY_CONFIG;
static char BCODE gszRegValSingleModeDMA[]    = REGSTR_VAL_SINGLEMODEDMA ;
static char BCODE gszRegValAcceptCloseRates[] = REGSTR_VAL_ACCEPTCLOSERATES ;
static char BCODE gszRegValSoundBlasterEmulation[] = REGSTR_VAL_SBEMULATION ;
static char BCODE gszRegValDo11kHzMCE[]       = REGSTR_VAL_DO11KHZMCE ;

extern char gszConfig[] ;

typedef RETERR (WINAPI *FNDICHANGESTATE)( LPDEVICE_INFO lpdi, DWORD dwStateChange, DWORD dwFlags, LPARAM lParam ) ;

#if defined(SW32)
#include "mydebug.h"
static char BCODE szGENMID[]				= "c:\\sw32\\genmid.ld";
static char BCODE gszRegValueDspFile[]  	= "DspFile";
static char BCODE gszRegValueCdRomPort[]  	= "CdRomPort";
static char BCODE gszRegValueCdRomIRQ[]  	= "CdRomIRQ";
static char BCODE gszRegValueCdRomDMA[]  	= "CdRomDMA";
static char BCODE gszRegValueSbPort[]  		= "SbPort";
static char BCODE gszRegValueSbIRQ[]  		= "SbIRQ";
static char BCODE gszRegValueSbDMA[]  		= "SbDMA";
static char BCODE gszRegValueMpuPort[]  	= "MpuPort";
static char BCODE gszRegValueMpuIRQ[]  		= "MpuIRQ";
static char BCODE gszRegValueGamePort[]  	= "EnableGamePort";
//	valid values for the settings: default followed by V1 to Vn
static int BCODE ganCdRomPort[] =	{ 0x000, 0x300, 0x310, 0x320, 0x330, 0x340, 0x360 };
static int BCODE ganCdRomIRQ[] =	{ -1, 9, 3, 5, 6, 7, 9, 10, 11 };
static int BCODE ganCdRomDMA[] =	{ -1, -1, 1, 3 };
static int BCODE ganSbPort[] =		{ 0x220, 0x220, 0x240 };
static int BCODE ganSbIRQ[] = 		{ 7, 3, 5, 7 };
static int BCODE ganSbDMA[] = 		{ 1, 1 };
static int BCODE ganMpuPort[] = 	{ 0x000, 0x330, 0x300, 0x330 };
static int BCODE ganMpuIRQ[] = 		{ -1, 5, 3, 4, 5, 7, 9 };
static int BCODE ganGamePort[] = 	{ 1, 0, 1 };

#define NUM_ELEMENTS(array)		(sizeof(array) / sizeof(array[0]))
#define STR_LEN(string)			(NUM_ELEMENTS(string) - 1)
#define HEX		16
#define DECIMAL	10

// helper macro used in DrvLoadUserConfig
#define GET_SW32_REG_NUM(name)								\
   cbLen = sizeof(nValue);									\
   cr = CM_Read_Registry_Value( dnDevNode,					\
                                gszRegKeyConfig,			\
                                gszRegValue##name,			\
                                REG_BINARY,					\
                                &nValue,					\
                                &cbLen,						\
                                CM_REGISTRY_SOFTWARE );		\
   if (CR_SUCCESS != cr)									\
       nValue = gan##name[0];								\
   pDrvData->n##name = nValue;

// helper macro used in DrvSaveUserConfig
#define SET_SW32_REG_NUM(name)								\
    nValue = pDrvData->n##name;								\
    CM_Write_Registry_Value( pDrvData->lpdi->dnDevnode,		\
                             gszRegKeyConfig,				\
                             gszRegValue##name,				\
                             REG_BINARY,					\
                             &nValue,						\
                             sizeof(nValue),				\
                             CM_REGISTRY_SOFTWARE);


// helper macro used in InitDrvPropertiesDlg
#define INIT_COMBO_BOX(id_control, name, base)				\
   InitComboBox(hDlg, id_control, gan##name, NUM_ELEMENTS(gan##name), pDrvData->n##name, base)


void InitComboBox (HWND hDlg, int nControlID, LPINT pnValues, int nNum, int nDefault, int nBase)
{
	char szNum[20];
	int i;
	HWND hCombo = GetDlgItem(hDlg, nControlID);
	LPSTR pszFormat = ((nBase == DECIMAL) ? "%3d" : "%03X");

	ASSERT(hCombo);
	ComboBox_ResetContent(hCombo);
	for (i = 1; i < nNum; i++) {
		if (pnValues[i] == -1) {
			szNum[0] = '\0';
			}
		else {
			wsprintf(szNum, pszFormat, pnValues[i]);
			}
		ComboBox_AddString(hCombo, szNum);
		}
	if (nDefault == -1) {
		szNum[0] = '\0';
		}
	else {
		wsprintf(szNum, pszFormat, nDefault);
		}
	ComboBox_SetText(hCombo, szNum);
}

LPSTR SkipSpaces (LPSTR pszText)
{
	while (*pszText == ' ') {
		pszText++;
		}
	return (pszText);
}

__inline BOOL IsDigit (char c)	{ return ((c >= '0') && (c <= '9')); }

int StringToInt (LPSTR pszNum)
{
	int nNum = 0;
	int bNegative = FALSE;

	pszNum = SkipSpaces(pszNum);
	if (*pszNum == '-') {
		bNegative = TRUE;
		pszNum++;
		}
	while (IsDigit(*pszNum)) {
		nNum *= 10;
		nNum += ((*pszNum) - '0');
		pszNum++;
		}
	if (bNegative) {
		nNum = -nNum;
		}

	return (nNum);
}

__inline BOOL IsHexDigit (char c)	
{
	return (IsDigit(c) || ((toupper(c) >= 'A') && (toupper(c) <= 'F')));
}

// TODO: reconcile with StringToInt (and/or use library routine)
int HexStringToInt (LPSTR pszNum)
{
	int nNum = 0;
	pszNum = SkipSpaces(pszNum);
	while (IsHexDigit(*pszNum)) {
		nNum *= 16;
		nNum += (IsDigit(*pszNum) ? (*pszNum - '0') : (10 + toupper(*pszNum) - 'A'));
		pszNum++;
		}
	return (nNum);
}

int GetComboBoxValue (HWND hDlg, int nControlID, int nBase)
{
	char szNum[20];
	LPSTR pszNum;
	int nValue;
	HWND hCombo = GetDlgItem(hDlg, nControlID);
	ASSERT(hCombo);

	ComboBox_GetText(hCombo, szNum, STR_LEN(szNum));
	pszNum = SkipSpaces(szNum);
	if (*pszNum == '\0') {
		nValue = -1;
		}
	else {
		nValue = (nBase == DECIMAL) ? StringToInt(pszNum) : HexStringToInt(pszNum);
		}

	return (nValue);
}

#define GET_COMBO_BOX_VALUE(id_control, name, base)			\
	nValue = GetComboBoxValue(hDlg, id_control, base);		\
    if (pDrvData->n##name != nValue) {						\
         fChanged = TRUE ;									\
         pDrvData->n##name = nValue;						\
      	 }

#endif


//--------------------------------------------------------------------------
//  
//  VOID DrvLoadUserConfig
//  
//  Description:
//      Reads the user configurable settings from the registry
//  
//  Parameters:
//
//  Return (VOID):
//  
//  
//--------------------------------------------------------------------------

VOID NEAR DrvLoadUserConfig
(
    DEVNODE         dnDevNode,
    LPDRVDATA       pDrvData
)
{

   BYTE       b ;
   DWORD      cbLen ;
   CONFIGRET  cr ;

   DPF(3, "DrvLoadUserConfig");
   //
   // Accept close sampling rates
   //

   if (NULL != pDrvData -> lpdi)
      dnDevNode = pDrvData -> lpdi -> dnDevnode ;

   cbLen = sizeof( b ) ;
   cr = CM_Read_Registry_Value( dnDevNode,
                                gszRegKeyConfig,
                                gszRegValAcceptCloseRates,
                                REG_BINARY,
                                &b,
                                &cbLen,
                                CM_REGISTRY_SOFTWARE ) ;
   if (CR_SUCCESS != cr)
      b = DEFAULT_ACCEPTCLOSERATES ;

   pDrvData -> fAcceptCloseRates = (b != 0) ;
   
   //
   // Single Mode DMA
   //

   cbLen = sizeof( b ) ;
   cr = CM_Read_Registry_Value( dnDevNode,
                                gszRegKeyConfig,
                                gszRegValSingleModeDMA,
                                REG_BINARY,
                                &b,
                                &cbLen,
                                CM_REGISTRY_SOFTWARE ) ;
   if (CR_SUCCESS != cr)
      b = DEFAULT_SINGLEMODEDMA ;

   pDrvData -> fSingleModeDMA = (b != 0) ;

   //
   // Sound Blaster emulation
   //

   if (pDrvData -> pszTemplate == IDD_PROPDLG_SB)
   {
      cbLen = sizeof( b ) ;
      cr = CM_Read_Registry_Value( dnDevNode,
                                 gszRegKeyConfig,
                                 gszRegValSoundBlasterEmulation,
                                 REG_BINARY,
                                 &b,
                                 &cbLen,
                                 CM_REGISTRY_SOFTWARE ) ;
      if (CR_SUCCESS != cr)
         b = DEFAULT_SBEMULATION ;

      pDrvData -> fSBEmulation = (b != 0) ;
   }

   //
   // Do extra MCE for 11KHz sample rates
   //

   cbLen = sizeof( b ) ;
   cr = CM_Read_Registry_Value( dnDevNode,
                                gszRegKeyConfig,
                                gszRegValDo11kHzMCE,
                                REG_BINARY,
                                &b,
                                &cbLen,
                                CM_REGISTRY_SOFTWARE ) ;
   if (CR_SUCCESS != cr)
       b = DEFAULT_DO11KHZMCE ;

   pDrvData -> fDo11kHzMCE = (b != 0) ;
   
#if defined(SW32)
{
	int nValue;
	cbLen = sizeof(pDrvData->szDspFile) - 1;
	cr = CM_Read_Registry_Value( dnDevNode,
                                gszRegKeyConfig,
                                gszRegValueDspFile,
                                REG_SZ,
                                pDrvData->szDspFile,
                                &cbLen,
                                CM_REGISTRY_SOFTWARE ) ;
	if (CR_SUCCESS != cr) {
		lstrcpy(pDrvData->szDspFile, szGENMID);
		}
	GET_SW32_REG_NUM(CdRomPort);
	GET_SW32_REG_NUM(CdRomIRQ);
	GET_SW32_REG_NUM(CdRomDMA);
	GET_SW32_REG_NUM(SbPort);
	GET_SW32_REG_NUM(SbIRQ);
	GET_SW32_REG_NUM(SbDMA);
	GET_SW32_REG_NUM(MpuPort);
	GET_SW32_REG_NUM(MpuIRQ);
	GET_SW32_REG_NUM(GamePort);
}
#endif

   return ;

} // DrvLoadUserConfig()

//--------------------------------------------------------------------------
//  
//  VOID DrvSaveUserConfig
//  
//  Description:
//     Writes the user configurable settings to the registry
//  
//  Parameters:
//  
//  Return (VOID):
//  
//  
//--------------------------------------------------------------------------

VOID NEAR DrvSaveUserConfig
(
    PDRVDATA        pDrvData
)
{
    BYTE b ;
    
    DPF(3, "DrvSaveUserConfig");
    //
    //  Save fAcceptCloseRates
    //

    b = pDrvData -> fAcceptCloseRates ;
    CM_Write_Registry_Value( pDrvData -> lpdi -> dnDevnode,
                             gszRegKeyConfig,
                             gszRegValAcceptCloseRates,
                             REG_BINARY,
                             &b,
                             sizeof( b ),
                             CM_REGISTRY_SOFTWARE ) ;

    //
    //  Save fSingleModeDMA
    //

    b = pDrvData -> fSingleModeDMA ;

    CM_Write_Registry_Value( pDrvData -> lpdi -> dnDevnode,
                             gszRegKeyConfig,
                             gszRegValSingleModeDMA,
                             REG_BINARY,
                             &b,
                             sizeof( b ),
                             CM_REGISTRY_SOFTWARE ) ;

    //
    //  Save fSBEmulation
    //

    if (pDrvData -> pszTemplate == IDD_PROPDLG_SB)
    {
      b = pDrvData -> fSBEmulation ;

      CM_Write_Registry_Value( pDrvData -> lpdi -> dnDevnode,
                              gszRegKeyConfig,
                              gszRegValSoundBlasterEmulation,
                              REG_BINARY,
                              &b,
                              sizeof( b ),
                              CM_REGISTRY_SOFTWARE ) ;
    }

    //
    //  Save 11kHz MCE stuff
    //

    b = pDrvData -> fDo11kHzMCE ;

    CM_Write_Registry_Value( pDrvData -> lpdi -> dnDevnode,
                             gszRegKeyConfig,
                             gszRegValDo11kHzMCE,
                             REG_BINARY,
                             &b,
                             sizeof( b ),
                             CM_REGISTRY_SOFTWARE ) ;

#if defined(SW32)
{
	DWORD cbLen = 1 + strlen(pDrvData->szDspFile);
	int nValue;
	CM_Write_Registry_Value( pDrvData -> lpdi -> dnDevnode,
                            gszRegKeyConfig,
                            gszRegValueDspFile,
                            REG_SZ,
                            pDrvData->szDspFile,
                            cbLen,
                            CM_REGISTRY_SOFTWARE ) ;
	SET_SW32_REG_NUM(CdRomPort);
	SET_SW32_REG_NUM(CdRomIRQ);
	SET_SW32_REG_NUM(CdRomDMA);
	SET_SW32_REG_NUM(SbPort);
	SET_SW32_REG_NUM(SbIRQ);
	SET_SW32_REG_NUM(SbDMA);
	SET_SW32_REG_NUM(MpuPort);
	SET_SW32_REG_NUM(MpuIRQ);
	SET_SW32_REG_NUM(GamePort);
}
#endif

    return ;

} // DrvSaveUserConfig()

//--------------------------------------------------------------------------
//  
//  VOID InitDrvPropertiesDlg
//  
//  Description:
//  
//  
//  Parameters:
//      HWND hDlg
//  
//      PDRVDATA pDrvData
//  
//  Return (VOID):
//  
//  
//--------------------------------------------------------------------------

VOID NEAR InitDrvPropertiesDlg
(
    HWND            hDlg,
    PDRVDATA        pDrvData
)
{
   DPF(3, "InitDrvPropertiesDlg");
   DrvLoadUserConfig( pDrvData -> lpdi -> dnDevnode, pDrvData ) ;
   
   SendDlgItemMessage( hDlg, IDC_DRVPROP_ACCEPTCLOSERATES,
                       BM_SETCHECK, pDrvData -> fAcceptCloseRates, 0L ) ;
   SendDlgItemMessage( hDlg, IDC_DRVPROP_SINGLEMODEDMA,
                       BM_SETCHECK, pDrvData -> fSingleModeDMA, 0L ) ;

   if (pDrvData -> pszTemplate == IDD_PROPDLG_SB)
      SendDlgItemMessage( hDlg, IDC_DRVPROP_ENABLESB,
                          BM_SETCHECK, pDrvData -> fSBEmulation, 0L ) ;
#if defined(SW32)
{
	UINT uTemplate = HIWORD(GetWindowLong(hDlg, DWL_USER));
   	if (uTemplate == (UINT)IDD_PROPDLG_SW32) {
		SendDlgItemMessage(hDlg, IDC_DSP_FILE, WM_SETTEXT, 0, (LPARAM)((LPSTR)pDrvData->szDspFile));
   		INIT_COMBO_BOX(IDC_CD_PORT, CdRomPort, HEX);
   		INIT_COMBO_BOX(IDC_CD_IRQ, CdRomIRQ, DECIMAL);
   		INIT_COMBO_BOX(IDC_CD_DMA, CdRomDMA, DECIMAL);
   		INIT_COMBO_BOX(IDC_SB_PORT, SbPort, HEX);
   		INIT_COMBO_BOX(IDC_SB_IRQ, SbIRQ, DECIMAL);
   		INIT_COMBO_BOX(IDC_SB_DMA, SbDMA, DECIMAL);
   		INIT_COMBO_BOX(IDC_MPU_PORT, MpuPort, HEX);
   		INIT_COMBO_BOX(IDC_MPU_IRQ, MpuIRQ, DECIMAL);
   		Button_SetCheck(GetDlgItem(hDlg, IDC_GAME_PORT), pDrvData->nGamePort);
		}
}
#endif

} // InitDrvPropertiesDlg()


//--------------------------------------------------------------------------
//  
//  VOID ApplyDrvPropertiesDlg
//  
//  Description:
//  
//  
//  Parameters:
//      HWND hDlg
//  
//      PDRVDATA pDrvData
//  
//  Return (VOID):
//  
//  
//--------------------------------------------------------------------------

VOID NEAR ApplyDrvPropertiesDlg
(
    HWND            hDlg,
    PDRVDATA        pDrvData
)
{
   BOOL  fChanged, fSetting ;
   DPF(3, "ApplyDrvPropertiesDlg");

   fChanged = FALSE ;


   fSetting =
      (BOOL) SendDlgItemMessage( hDlg, IDC_DRVPROP_ACCEPTCLOSERATES,
                                 BM_GETCHECK, 0, 0L );
   if (pDrvData -> fAcceptCloseRates != fSetting)
   {
      fChanged = TRUE ;
      pDrvData -> fAcceptCloseRates = fSetting ;
   }

   fSetting =
      (BOOL) SendDlgItemMessage( hDlg, IDC_DRVPROP_SINGLEMODEDMA,
                                 BM_GETCHECK, 0, 0L ) ;

   if (pDrvData -> fSingleModeDMA != fSetting)
   {
      fChanged = TRUE ;
      pDrvData -> fSingleModeDMA = fSetting ;
   }

   if (pDrvData -> pszTemplate == IDD_PROPDLG_SB)
   {
      fSetting =
         (BOOL) SendDlgItemMessage( hDlg, IDC_DRVPROP_ENABLESB,
                                    BM_GETCHECK, 0, 0L ) ;

      if (pDrvData -> fSBEmulation != fSetting)
      {
         fChanged = TRUE ;
         pDrvData -> fSBEmulation = fSetting ;
      }
   }

#if defined(SW32)
{
	UINT uTemplate = HIWORD(GetWindowLong(hDlg, DWL_USER));
   	if (uTemplate == (UINT)IDD_PROPDLG_SW32) {
   		int nValue;
   		char szTemp[MAX_PATH];

   		SendDlgItemMessage(hDlg, IDC_DSP_FILE, WM_GETTEXT, PATH_MAX, (LPARAM)((LPSTR)szTemp));
		if (strcmp(szTemp, pDrvData->szDspFile) != 0) {
			fChanged = TRUE;
			strcpy(pDrvData->szDspFile, szTemp);
   	   		}
   		GET_COMBO_BOX_VALUE(IDC_CD_PORT, CdRomPort, HEX);
   		GET_COMBO_BOX_VALUE(IDC_CD_IRQ, CdRomIRQ, DECIMAL);
   		GET_COMBO_BOX_VALUE(IDC_CD_DMA, CdRomDMA, DECIMAL);
   		GET_COMBO_BOX_VALUE(IDC_SB_PORT, SbPort, HEX);
   		GET_COMBO_BOX_VALUE(IDC_SB_IRQ, SbIRQ, DECIMAL);
   		GET_COMBO_BOX_VALUE(IDC_SB_DMA, SbDMA, DECIMAL);
   		GET_COMBO_BOX_VALUE(IDC_MPU_PORT, MpuPort, HEX);
   		GET_COMBO_BOX_VALUE(IDC_MPU_IRQ, MpuIRQ, DECIMAL);
		fSetting = Button_GetCheck(GetDlgItem(hDlg, IDC_GAME_PORT));
    	if (pDrvData->nGamePort != fSetting) {
        	fChanged = TRUE;
         	pDrvData->nGamePort = fSetting;
      	 	}
   
		}
}
#endif

   if (fChanged)
   {
      HINSTANCE        hMod ;
      FNDICHANGESTATE  pDiChangeState ;
      UINT             uLastErrorMode ;

      //
      // restart the device with the new configuration settings
      // 

      DrvSaveUserConfig( pDrvData ) ;

      uLastErrorMode = SetErrorMode( SEM_NOOPENFILEERRORBOX ) ;
      if (HINSTANCE_ERROR < (hMod = LoadLibrary( "SETUPX.DLL" )))
      {
         pDiChangeState = 
            (FNDICHANGESTATE) GetProcAddress( hMod, "DiChangeState" ) ;
         if (pDiChangeState)
            pDiChangeState( pDrvData -> lpdi, DICS_PROPCHANGE, 
                            DICS_FLAG_GLOBAL, 0 ) ;
         FreeLibrary( hMod ) ;
      }
      SetErrorMode( uLastErrorMode) ;
   }

} // ApplyDrvPropertiesDlg()

//--------------------------------------------------------------------------
//  
//  BOOL DrvPropertiesDlgProc
//  
//  Description:
//  
//  
//  Parameters:
//      HWND hDlg
//  
//      UINT uMsg
//  
//      WPARAM wParam
//  
//      LPARAM lParam
//  
//  Return (BOOL):
//  
//--------------------------------------------------------------------------

BOOL CALLBACK _loadds DrvPropertiesDlgProc
(
    HWND            hDlg,
    UINT            uMsg,
    WPARAM          wParam,
    LPARAM          lParam
)
{
   PDRVDATA  pDrvData ;
#if defined(SW32)
   DEXEC(5, DEBUG_OUT4("DrvPropertiesDlgProc(%04X, %04X, %04X, %08X)", hDlg, uMsg, wParam, lParam));
#endif

   pDrvData = (PDRVDATA) GetWindowLong( hDlg, DWL_USER ) ;

   switch(uMsg)
   {
      case WM_INITDIALOG:
         pDrvData = (PDRVDATA) (((LPPROPSHEETPAGE) lParam) -> lParam) ;
         SetWindowLong( hDlg, DWL_USER, MAKELONG( pDrvData, 0 ) ) ;
#if defined(SW32)
{
		 UINT uTemplate = HIWORD(((LPPROPSHEETPAGE) lParam)->lParam);
         SetWindowLong(hDlg, DWL_USER, MAKELONG(pDrvData, uTemplate));
}
#endif
         InitDrvPropertiesDlg( hDlg, pDrvData ) ;
         break;
         
      case WM_DESTROY:
#if defined(SW32)
{
		 UINT uTemplate = HIWORD(GetWindowLong(hDlg, DWL_USER));
   		 if (uTemplate == (UINT)IDD_PROPDLG_SW32) {
		 	break;
			}
}
#endif
         LocalFree( (HLOCAL) pDrvData ) ;
         break ;

      case WM_COMMAND:
      {
         switch (wParam)
         {
            case IDC_DRVPROP_SETDEFAULTS:
               SendDlgItemMessage( hDlg, IDC_DRVPROP_ACCEPTCLOSERATES,
                                   BM_SETCHECK,
                                   DEFAULT_ACCEPTCLOSERATES, 0L ) ;
               SendDlgItemMessage( hDlg, IDC_DRVPROP_SINGLEMODEDMA,
                                   BM_SETCHECK,
                                   DEFAULT_SINGLEMODEDMA, 0L ) ;
               SendDlgItemMessage( hDlg, IDC_DRVPROP_ENABLESB,
                                   BM_SETCHECK,
                                   DEFAULT_SBEMULATION, 0L ) ;
#if 0 && defined(SW32)
   			   SendDlgItemMessage(hDlg, IDC_DSP_FILE, WM_SETTEXT, 0, (LPARAM)((LPSTR)szGENMID));
#endif
               break ;
         }
         break ;
      }
      
      case WM_NOTIFY:
         switch (((NMHDR FAR *)lParam)->code)
         {
            case PSN_APPLY:
               ApplyDrvPropertiesDlg( hDlg, pDrvData ) ;
               break ;
#if defined(SW32)
			default:
			   DEBUG_OUT1("WM_NOTIFY code %04X not processed", ((NMHDR FAR *)lParam)->code);
			   break;
#endif
         }
         break ;

      case WM_CONTEXTMENU:
         WinHelp( (HWND) wParam, (LPSTR) cszHelpFile, HELP_CONTEXTMENU,
                  (DWORD) (LPSTR) aHelpIds ) ;
         return TRUE ;

      case WM_HELP:
      {
         LPHELPINFO lphi = (LPVOID) lParam;
         WinHelp( lphi -> hItemHandle, (LPSTR)cszHelpFile, HELP_WM_HELP,
                  (DWORD) (LPSTR) aHelpIds ) ;
         return TRUE ;
      }
      
      default:
         return FALSE ;
   }
   return TRUE ;

} // DrvPropertiesDlgProc()

//--------------------------------------------------------------------------
//  
//  BOOL DrvEnumPropPages
//  
//  Description:
//  
//  
//  Parameters:
//      LPDEVICE_INFO lpdi
//  
//      LPFNADDPROPSHEETPAGE pfn
//  
//      LPARAM lParam
//  
//  Return (BOOL):
//
//  
//--------------------------------------------------------------------------

BOOL WINAPI _loadds DrvEnumPropPages
(
    LPDEVICE_INFO           lpdi,
    LPFNADDPROPSHEETPAGE    pfn,
    LPARAM                  lParam
)
{
   PDRVDATA           pDrvData ;
   PROPSHEETPAGE      psp ;
   HPROPSHEETPAGE     hpage ;
   PHARDWAREINSTANCE  phwi ;
   DPF(3, "DrvEnumPropPages");

   if (NULL == (phwi = DevNodeToHardwareInstance( lpdi -> dnDevnode )))
      return FALSE ;

   //  Allocate a per-instance dialog data structure.

   if (NULL != 
         (pDrvData = (PDRVDATA) LocalAlloc( LPTR, sizeof( DRVDATA ) )))
   {

      psp.dwSize = sizeof( psp ) ;
      psp.dwFlags = PSP_DEFAULT ;
      psp.hInstance = ghModule ;

      //
      // Only use SB emulation settings when hardware 
      // is not present (because we virtualize a Sound Blaster
      // in this case).
      //

      if (phwi -> wFlags & SSI_FLAG_HWSB)
         psp.pszTemplate = IDD_PROPDLG_NOSB ;
      else
         psp.pszTemplate = IDD_PROPDLG_SB ;
      psp.pfnDlgProc = DrvPropertiesDlgProc;
      psp.lParam = MAKELPARAM( pDrvData, 0 ) ;

      pDrvData -> lpdi = lpdi ;
      pDrvData -> pszTemplate = psp.pszTemplate ;

      hpage = CreatePropertySheetPage( &psp ) ;
      if (hpage)
      {
         if (!pfn(hpage, lParam))
            DestroyPropertySheetPage(hpage) ;
      }

#if defined(SW32)
{
	HPROPSHEETPAGE     hpage_sw32;

	pDrvData->szDspFile[0] = '\0';
    psp.dwSize = sizeof( psp ) ;
    psp.dwFlags = PSP_DEFAULT ;
    psp.hInstance = ghModule ;
    psp.pszTemplate = IDD_PROPDLG_SW32;
    psp.pfnDlgProc = DrvPropertiesDlgProc;
    psp.lParam = MAKELPARAM(pDrvData, (UINT)IDD_PROPDLG_SW32) ;
	hpage_sw32 = CreatePropertySheetPage( &psp ) ;
	if (hpage_sw32) {
		if (!pfn(hpage_sw32, lParam)) {
			DestroyPropertySheetPage(hpage);
            DestroyPropertySheetPage(hpage_sw32);
			}
      }
}
#endif
   }
   return( TRUE ) ;

} // DrvEnumPropPages()

//---------------------------------------------------------------------------
//  End of File: proppage.c
//---------------------------------------------------------------------------

